﻿using ShipyardBusinessLogic.OfficePackage.HelperModels;
using ShipyardBusinessLogic.OfficePackage;
using ShipyardContracts.BindingModels;
using ShipyardContracts.BusinessLogicsContracts;
using ShipyardContracts.SearchModels;
using ShipyardContracts.StoragesContracts;
using ShipyardContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShipyardBusinessLogic.BusinessLogics
{
    public class ReportLogic: IReportLogic
    {
        private readonly IDetailStorage _detailStorage;

        private readonly IShipStorage _shipStorage;

        private readonly IOrderStorage _orderStorage;

        private readonly AbstractSaveToExcel _saveToExcel;

        private readonly AbstractSaveToWord _saveToWord;

        private readonly AbstractSaveToPdf _saveToPdf;

        public ReportLogic(IShipStorage shipStorage, IDetailStorage detailStorage, IOrderStorage orderStorage,
            AbstractSaveToExcel saveToExcel, AbstractSaveToWord saveToWord, AbstractSaveToPdf saveToPdf)
        {
            _shipStorage = shipStorage;
            _detailStorage = detailStorage;
            _orderStorage = orderStorage;

            _saveToExcel = saveToExcel;
            _saveToWord = saveToWord;
            _saveToPdf = saveToPdf;
        }
        public List<ReportShipDetailViewModel> GetProductComponent()
        {
            var details = _detailStorage.GetFullList();

            var ships = _shipStorage.GetFullList();

            var list = new List<ReportShipDetailViewModel>();

            foreach (var detail in details)
            {
                var record = new ReportShipDetailViewModel
                {
                    DetailName = detail.DetailName,
                    Ships = new List<(string, int)>(),
                    TotalCount = 0
                };
                foreach (var ship in ships)
                {
                    if (ship.ShipDetails.ContainsKey(detail.Id))
                    {
                        record.Ships.Add(new(ship.ShipName, ship.ShipDetails[detail.Id].Item2));
                        record.TotalCount += ship.ShipDetails[detail.Id].Item2;
                    }
                }

                list.Add(record);
            }

            return list;
        }
        public List<ReportOrdersViewModel> GetOrders(ReportBindingModel model)
        {
            return _orderStorage.GetFilteredList(new OrderSearchModel { DateFrom = model.DateFrom, DateTo = model.DateTo })
                    .Select(x => new ReportOrdersViewModel
                    {
                        Id = x.Id,
                        DateCreate = x.DateCreate,
                        ShipName = x.ShipName,
                        Sum = x.Sum
                    })
                    .ToList();
        }

        public void SaveComponentsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список деталей",
                Details = _detailStorage.GetFullList()
            });
        }

        public void SaveProductComponentToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Список деталей",
                ShipDetails = GetProductComponent()
            });
        }

        public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Список заказов",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Orders = GetOrders(model)
            });
        }
    }
}
